<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:stsuuser="urn:ibm:names:ITFIM:1.0:stsuuser">

        <xsl:strip-space elements="*" />
        <xsl:output method="xml" version="1.0" encoding="utf-8" indent="yes" />

        <!--
                Initially we start with a copy of the document.
        -->
        <xsl:template match="@* | node()">
                <xsl:copy>
                        <xsl:apply-templates select="@* | node()" />
                </xsl:copy>
        </xsl:template>

        <!--
                This template replaces the AttributeList with one containing only the subset
                of attributes we are interested in (whilst modifying their data type) and
                adds a tagvalue_email attribute which is the current principal name value
                (from the SAML assertion). When copying attributes that we are interested
                in we also map their names as follows:
                commonName->tagvalue_name
                ssn->tagvalue_ssn
                role->tagvalue_role (only this one can be multi-valued)

                Additional attributes are added to indicate this is a single signon, and the federation name.
            These attributes are useful for providing user friendly look and feel for SSO'd users.
        -->
        <xsl:template match="//stsuuser:AttributeList">
                <stsuuser:AttributeList>

                        <!-- The tagvalue_name attribute -->
                        <stsuuser:Attribute name="tagvalue_name" type="urn:ibm:names:ITFIM:5.1:accessmanager">
                                <stsuuser:Value>
                                        <xsl:value-of select="//stsuuser:AttributeList/stsuuser:Attribute[@name='commonName'][@type='http://example.com/federation/v1/commonName']/stsuuser:Value" />
                                </stsuuser:Value>
                        </stsuuser:Attribute>

                        <!-- The tagvalue_ssn attribute -->
                        <stsuuser:Attribute name="tagvalue_ssn" type="urn:ibm:names:ITFIM:5.1:accessmanager">
                                <stsuuser:Value>
                                        <xsl:value-of select="//stsuuser:AttributeList/stsuuser:Attribute[@name='ssn'][@type='http://example.com/federation/v1/namevalue']/stsuuser:Value" />
                                </stsuuser:Value>
                        </stsuuser:Attribute>

                        <!-- The tagvalue_role attribute (can be multi-valued) -->
                        <stsuuser:Attribute name="tagvalue_role" type="urn:ibm:names:ITFIM:5.1:accessmanager">
                                <xsl:for-each select="//stsuuser:AttributeList/stsuuser:Attribute[@name='role'][@type='http://example.com/federation/v1/role']/stsuuser:Value">
                                        <stsuuser:Value>
                                                <xsl:apply-templates />
                                        </stsuuser:Value>
                                </xsl:for-each>
                        </stsuuser:Attribute>

                        <!-- The tagvalue_email attribute -->
                        <stsuuser:Attribute name="tagvalue_email" type="urn:ibm:names:ITFIM:5.1:accessmanager">
                                <stsuuser:Value>
                                        <xsl:value-of select="//stsuuser:Principal/stsuuser:Attribute[@name='name']/stsuuser:Value" />
                                </stsuuser:Value>
                        </stsuuser:Attribute>

			<!-- The tagvalue_sso attribute -->
            <stsuuser:Attribute name="tagvalue_sso" type="urn:ibm:names:ITFIM:5.1:accessmanager">
                    <stsuuser:Value>isSingleSignOn</stsuuser:Value>
            </stsuuser:Attribute>

			<!-- The tagvalue_fedname attribute -->
            <stsuuser:Attribute name="tagvalue_fedname" type="urn:ibm:names:ITFIM:5.1:accessmanager">
                    <stsuuser:Value>samlfed</stsuuser:Value>
            </stsuuser:Attribute>

		</stsuuser:AttributeList>
	</xsl:template>

	<!--
		This will replace the principal name (which was the email address in
		the SAML assertion) with the user "me_guest".
	-->
	<xsl:template match="//stsuuser:Principal/stsuuser:Attribute[@name='name']">
		<stsuuser:Attribute name="name" type="urn:ibm:names:ITFIM:5.1:accessmanager">
            <stsuuser:Value>me_guest</stsuuser:Value>
            </stsuuser:Attribute>
	</xsl:template>
</xsl:stylesheet>

