# @(#)21  1.4 src/cmdline/staging/scripts/saml20_utils.py, amws.cmdline, fim611, 070402a 3/28/07 16:48:48
# (C) Copyright International Business Machines Corporation 2005,2007. All 
# rights reserved.  U.S. Government Users Restricted Rights -- Use, duplication
# or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

from fim_base import FIM_TRUE, FIM_FALSE, print_dictionary, read_properties_file
from com.tivoli.am.fim.middlemanager.ssps import SAML20Service, SAML20IndexedService
from com.tivoli.am.fim.middlemanager.ssps import Federation, FederationManager
from java.util import ArrayList
from java.lang import String
import fed_utils

# SAML 2.0 Binding Constants
BINDING_NONE = -1
BINDING_HTTP_REDIRECT = 1
BINDING_HTTP_POST = 2
BINDING_HTTP_ARTIFACT = 3
BINDING_SOAP = 4
BINDING_PA0S = 5

# IY96453
# Exceptions
ProfileSpecErr="Profile specification must be a Python dictionary with valid \
binding, or \"Disabled\" or null (\"\") to disable the binding."
# IY96453

http_bindings = [BINDING_HTTP_REDIRECT, BINDING_HTTP_POST, BINDING_HTTP_ARTIFACT]
soap_bindings = [BINDING_SOAP]
paos_bindings = [BINDING_PA0S]

# Profiles common to all roles
saml20_profiles = ["ArtifactResolutionProfile", "ManageNameIdProfile", \
"SingleLogoutProfile"]

# SAML 2.0 Self profiles (for both IP and SP)
saml20_profiles_self=[]

# SAML 2.0 self identity provider (IP) profiles 
saml20_profiles_self_ip=["SingleSignOnProfile"]

# SAML 2.0 self service provider (SP) profiles
saml20_profiles_self_sp=["AssertionConsumerProfile"]

# SAML 2.0 partner profiles: Self=IP, Partner=SP
saml20_profiles_partner_ip=["AssertionConsumerProfile"]

# SAML 2.0 partner profiles: Self=SP, Partner=IP
saml20_profiles_partner_sp=["SingleSignOnProfile"]

# Profiles not exposed to in the ISC FIM admin console are listed below.
# These profiles are disabled for now in order to align with the console.
#saml20_profiles_self=["ForceAuthenticationProfile"]
#saml20_profiles_self_ip=["CommonDomainCookieWriterProfile"]
#saml20_profiles_self_sp=["AuthDecisionProfile", "CommonDomainCookieReaderProfile"]
#saml20_profiles_partner_ip=["AuthDecisionProfile"]

def get_saml20_supported_profiles(role, isPartner):
    if role=="ip" and not isPartner:
        return saml20_profiles + saml20_profiles_self + saml20_profiles_self_ip
    if role=="sp" and not isPartner:
        return saml20_profiles + saml20_profiles_self + saml20_profiles_self_sp
    if role=="ip" and isPartner:
        return saml20_profiles + saml20_profiles_partner_ip
    if role=="sp" and isPartner:
        return saml20_profiles + saml20_profiles_partner_sp 
    return None

def binding_to_string(binding):
    if binding==BINDING_NONE:
        return None
    elif binding==BINDING_HTTP_REDIRECT:
        return "HTTPRedirect"
    elif binding==BINDING_HTTP_POST:
        return "HTTPPost"
    elif binding==BINDING_HTTP_ARTIFACT:
        return "HTTPArtifact"
    elif binding==BINDING_SOAP:
        return "SOAP"
    elif binding==BINDING_PA0S:
        return "PAOS"

def string_to_binding(bstring):
    if bstring==None:
        return BINDING_NONE
    elif bstring=="HTTPRedirect" :
        return BINDING_HTTP_REDIRECT
    elif bstring=="HTTPPost" :
        return BINDING_HTTP_POST
    elif bstring=="HTTPArtifact" :
        return BINDING_HTTP_ARTIFACT
    elif bstring=="SOAP" :
        return BINDING_SOAP
    elif bstring=="PAOS" :
        return BINDING_PA0S

def get_service_values(svc_list, isSSOProfile):
    svc_values={}
    bindings = []
    for svc in svc_list:
        binding = svc.getBinding()
        bindings.append(binding_to_string(binding))
        if binding in http_bindings:
            svc_values["HTTPEndpoint"]=svc.getEndpoint()
            svc_values["ResponseLocation"]=svc.getResponseLocation()
        if binding in soap_bindings:
            svc_values["SOAPEndpoint"]=svc.getEndpoint()
            svc_values["ResponseLocation"]=svc.getResponseLocation()
        if binding in paos_bindings:
            svc_values["PAOSEndpoint"]=svc.getEndpoint()
            svc_values["ResponseLocation"]=svc.getResponseLocation()
    svc_values["Bindings"]=bindings
    
    return svc_values
        
def get_saml20_profile_properties(cfg, fed_role, isPartner):
    profiles = get_saml20_supported_profiles(fed_role, isPartner)
    props = {}
    isSSOProfile=FIM_FALSE
    for p in profiles:
        if p.find("SingleSignOn") >= 0:
            isSSOProfile=FIM_TRUE
        svc_list = eval("cfg.get%sServiceList()"%p[0:-7])
        if not svc_list:
            props[p] = "Disabled"
        else:
            svc_values = get_service_values(svc_list, isSSOProfile)
            props[p] = svc_values
        
    return props

def get_SAML20Service(svc_list, binding):
    for s in svc_list:
        if s.getBinding()==binding:
            return s
    return None

    
def get_SAML20IndexedService_list(cfg, props):
    indexed_svc_list = ArrayList()
    # IY96453
    # Make sure that the service is enabled and has bindings
    # If not, catch the exception and return the empty service list.
    try:
        if not props.has_key("Bindings"):
            # This is an error
            raise ProfileSpecErr
    except:
        # This might be okay, there are no bindings so return empty list
        # Check that the value is "Disabled"  or "", otherwise err.
        if props.lower() == 'disabled' or props == '':
            return indexed_svc_list
        else: 
            print 'Invalid profile property: ' + props + '.'
            raise ProfileSpecErr    
    # IY96453
    for b in props["Bindings"]:
        binding = string_to_binding(b)
        if binding in http_bindings:
            indexed_svc = SAML20IndexedService(props["HTTPEndpoint"], binding)
            indexed_svc.setResponseLocation(props["ResponseLocation"])
            indexed_svc_list.add(indexed_svc)
            continue
        if binding in soap_bindings:
            indexed_svc = SAML20IndexedService(props["SOAPEndpoint"], binding)
            indexed_svc.setResponseLocation(props["ResponseLocation"])
            indexed_svc_list.add(indexed_svc)
            continue
        if binding in paos_bindings:
            indexed_svc = SAML20IndexedService(props["PAOSEndpoint"], binding)
            indexed_svc.setResponseLocation(props["ResponseLocation"])
            indexed_svc_list.add(indexed_svc)
            continue
    return indexed_svc_list

def get_SAML20Service_list(cfg, props):
    svc_list = ArrayList()
    # IY94088, IY96453
    # Make sure that the service is enabled and has bindings
    # If not, catch the exception and return the empty service list.
    try:
        if not props.has_key("Bindings"):
            # This is an error
            raise ProfileSpecErr
    except:
        # This might be okay, there are no bindings so return empty list
        # Check that the value is "Disabled"  or "", otherwise err.
        if props.lower() == 'disabled' or props == '':
            return svc_list
        else: 
            print 'Invalid profile property: ' + props + '.'
            raise ProfileSpecErr    
    # IY94088, IY96453
    for b in props["Bindings"]:
        binding = string_to_binding(b)
        if binding in http_bindings:
            svc = SAML20Service(props["HTTPEndpoint"], binding)
            svc.setResponseLocation(props["ResponseLocation"])
            svc_list.add(svc)
        if binding in soap_bindings:
            svc = SAML20Service(props["SOAPEndpoint"], binding)
            svc.setResponseLocation(props["ResponseLocation"])
            svc_list.add(svc)
        if binding in paos_bindings:
            svc = SAML20Service(props["PAOSEndpoint"], binding)
            svc.setResponseLocation(props["ResponseLocation"])
            svc_list.add(svc)
            continue
    return svc_list



def set_saml20_profile_properties(cfg, fed_role, isPartner, user_props):
    profiles = get_saml20_supported_profiles(fed_role, isPartner)
    saml20_svc_list = ArrayList()
    for p in user_props.keys(): # cycle through user-supplied settings
        if p in profiles: # check to see if user is setting a supported profile
            saml20_svc_list = ArrayList()
            if p.find("ArtifactResolution")>=0:
                saml20_svc_list = get_SAML20IndexedService_list(cfg, user_props[p])
                cfg.setArtifactResolutionServiceList(saml20_svc_list)
            elif p.find("ManageNameId")>=0:
                saml20_svc_list = get_SAML20Service_list(cfg, user_props[p])
                cfg.setManageNameIdServiceList(saml20_svc_list)
            elif p.find("SingleLogout")>=0:
                saml20_svc_list = get_SAML20Service_list(cfg, user_props[p])
                cfg.setSingleLogoutServiceList(saml20_svc_list)
            elif p.find("SingleSignOn")>=0:
                saml20_svc_list = get_SAML20Service_list(cfg, user_props[p])
                cfg.setSingleSignOnServiceList(saml20_svc_list)
            elif p.find("AssertionConsumer")>=0:
                saml20_svc_list = get_SAML20IndexedService_list(cfg, user_props[p])
                cfg.setAssertionConsumerServiceList(saml20_svc_list)
    
if __name__ == 'main':

    if(len(sys.argv) == 2):
        isPartner=0
        props_file = sys.argv[0]
        fed_name = sys.argv[1]
    elif(len(sys.argv) == 3):
        isPartner=1
        props_file = sys.argv[0]
        fed_name = sys.argv[1]
        partner_name = sys.argv[2]
        
    # Get the federation object
    (fed, fed_mgr) = fed_utils.get_federation(fed_name, props_file)
    
    fed_role = fed.getSelfRoleStr()
    fed_protocol = fed.getSSOProtocolStr()

    if isPartner:
        partner = fed_utils.get_partner(fed, partner_name)
    else:
        partner = None

    if partner:
        isPartner = FIM_TRUE
        cfg = partner.getCfgParameters()
        # Get the endpoint properties for this partner
        svc_props = get_saml20_profile_properties(cfg, fed_role, isPartner)
    else:
        isPartner = FIM_FALSE
        cfg = fed.getCfgParameters()
        # Get the endpoint properties for this federation
        svc_props = get_saml20_profile_properties(cfg, fed_role, isPartner)

    print_dictionary(svc_props)
